# -*- coding: utf-8 -*-
"""
This serialiser will output an RDF Graph as a JSON-LD formatted document. See:

    http://json-ld.org/

Example usage::

    >>> from rdflib.plugin import register, Serializer
    >>> register('json-ld', Serializer, 'rdflib_jsonld.serializer', 'JsonLDSerializer')

    >>> from rdflib import Graph

    >>> testrdf = '''
    ... @prefix dc: <http://purl.org/dc/terms/> .
    ... <http://example.org/about>
    ...     dc:title "Someone's Homepage"@en .
    ... '''

    >>> g = Graph().parse(data=testrdf, format='n3')

    >>> print(g.serialize(format='json-ld', indent=4).decode())
    [
        {
            "@id": "http://example.org/about",
            "http://purl.org/dc/terms/title": [
                {
                    "@language": "en",
                    "@value": "Someone's Homepage"
                }
            ]
        }
    ]

"""

# NOTE: This code writes the entire JSON object into memory before serialising,
# but we should consider streaming the output to deal with arbitrarily large
# graphs.

import warnings

from rdflib.serializer import Serializer
from rdflib.graph import Graph
from rdflib.term import URIRef, Literal, BNode
from rdflib.namespace import RDF, XSD

from .context import Context, UNDEF
from .util import json
from .keys import CONTEXT, GRAPH, ID, VOCAB, LIST, SET, LANG

__all__ = ['JsonLDSerializer', 'from_rdf']


PLAIN_LITERAL_TYPES = set([XSD.boolean, XSD.integer, XSD.double, XSD.string])


class JsonLDSerializer(Serializer):
    def __init__(self, store):
        super(JsonLDSerializer, self).__init__(store)

    def serialize(self, stream, base=None, encoding=None, **kwargs):
        # TODO: docstring w. args and return value
        encoding = encoding or 'utf-8'
        if encoding not in ('utf-8', 'utf-16'):
            warnings.warn("JSON should be encoded as unicode. " +
                          "Given encoding was: %s" % encoding)

        context_data = kwargs.get('context')
        use_native_types = kwargs.get('use_native_types', False),
        use_rdf_type = kwargs.get('use_rdf_type', False)
        auto_compact = kwargs.get('auto_compact', False)

        indent = kwargs.get('indent', 2)
        separators = kwargs.get('separators', (',', ': '))
        sort_keys = kwargs.get('sort_keys', True)
        ensure_ascii = kwargs.get('ensure_ascii', False)

        obj = from_rdf(self.store, context_data, base,
                use_native_types, use_rdf_type,
                auto_compact=auto_compact)

        data = json.dumps(obj, indent=indent, separators=separators,
                          sort_keys=sort_keys, ensure_ascii=ensure_ascii)

        stream.write(data.encode(encoding, 'replace'))


def from_rdf(graph, context_data=None, base=None,
        use_native_types=False, use_rdf_type=False,
        auto_compact=False, startnode=None, index=False):
    # TODO: docstring w. args and return value
    # TODO: support for index and startnode

    if not context_data and auto_compact:
        context_data = dict(
            (pfx, unicode(ns))
            for (pfx, ns) in graph.namespaces() if pfx and
            unicode(ns) != u"http://www.w3.org/XML/1998/namespace")

    if isinstance(context_data, Context):
        context = context_data
        context_data = context.to_dict()
    else:
        context = Context(context_data, base=base)

    converter = Converter(context, use_native_types, use_rdf_type)
    result = converter.convert(graph)

    if converter.context.active:
        if isinstance(result, list):
            result = {context.get_key(GRAPH): result}
        result[CONTEXT] = context_data

    return result


class Converter(object):

    def __init__(self, context, use_native_types, use_rdf_type):
        self.context = context
        self.use_native_types = context.active or use_native_types
        self.use_rdf_type = use_rdf_type

    def convert(self, graph):
        # TODO: bug in rdflib dataset parsing (nquads et al):
        # plain triples end up in separate unnamed graphs (rdflib issue #436)
        if graph.context_aware:
            default_graph = Graph()
            graphs = [default_graph]
            for g in graph.contexts():
                if isinstance(g.identifier, URIRef):
                    graphs.append(g)
                else:
                    default_graph += g
        else:
            graphs = [graph]

        context = self.context

        objs = []
        for g in graphs:
            obj = {}
            graphname = None

            if isinstance(g.identifier, URIRef):
                graphname = context.shrink_iri(g.identifier)
                obj[context.id_key] = graphname

            nodes = self.from_graph(g)

            if not graphname and len(nodes) == 1:
                obj.update(nodes[0])
            else:
                if not nodes:
                    continue
                obj[context.graph_key] = nodes

            if objs and objs[0].get(context.get_key(ID)) == graphname:
                objs[0].update(obj)
            else:
                objs.append(obj)

        if len(graphs) == 1 and len(objs) == 1 and not self.context.active:
            default = objs[0]
            items = default.get(context.graph_key)
            if len(default) == 1 and items:
                objs = items
        elif len(objs) == 1 and self.context.active:
            objs = objs[0]

        return objs

    def from_graph(self, graph):
        nodemap = {}

        for s in set(graph.subjects()):
            ## only iri:s and unreferenced (rest will be promoted to top if needed)
            if isinstance(s, URIRef) or (isinstance(s, BNode)
                    and not any(graph.subjects(None, s))):
                self.process_subject(graph, s, nodemap)

        return nodemap.values()

    def process_subject(self, graph, s, nodemap):
        if isinstance(s, URIRef):
            node_id = self.context.shrink_iri(s)
        elif isinstance(s, BNode):
            node_id = s.n3()
        else:
            node_id = None

        #used_as_object = any(graph.subjects(None, s))
        if node_id in nodemap:
            return None

        node = {}
        node[self.context.id_key] = node_id
        nodemap[node_id] = node

        for p, o in graph.predicate_objects(s):
            self.add_to_node(graph, s, p, o, node, nodemap)

        return node

    def add_to_node(self, graph, s, p, o, s_node, nodemap):
        context = self.context

        if isinstance(o, Literal):
            datatype = unicode(o.datatype) if o.datatype else None
            language = o.language
            term = context.find_term(unicode(p), datatype, language=language)
        else:
            containers = [LIST, None] if graph.value(o, RDF.first) else [None]
            for container in containers:
                for coercion in (ID, VOCAB, UNDEF):
                    term = context.find_term(unicode(p), coercion, container)
                    if term:
                        break
                if term:
                    break

        node = None
        use_set = not context.active

        if term:
            p_key = term.name

            if term.type:
                node = self.type_coerce(o, term.type)
            elif term.language and o.language == term.language:
                node = unicode(o)
            elif context.language and (
                    term.language is None and o.language is None):
                node = unicode(o)

            if term.container == SET:
                use_set = True
            elif term.container == LIST:
                node = [self.type_coerce(v, term.type) or self.to_raw_value(graph, s, v, nodemap)
                        for v in self.to_collection(graph, o)]
            elif term.container == LANG and language:
                value = s_node.setdefault(p_key, {})
                values = value.get(language)
                node = unicode(o)
                if values:
                    if not isinstance(values, list):
                        value[language] = values = [values]
                    values.append(node)
                else:
                    value[language] = node
                return

        else:
            p_key = context.to_symbol(p)
            # TODO: for coercing curies - quite clumsy; unify to_symbol and find_term?
            key_term = context.terms.get(p_key)
            if key_term and (key_term.type or key_term.container):
                p_key = p
            if not term and p == RDF.type and not self.use_rdf_type:
                if isinstance(o, URIRef):
                    node = context.to_symbol(o)
                p_key = context.type_key

        if node is None:
            node = self.to_raw_value(graph, s, o, nodemap)

        value = s_node.get(p_key)
        if value:
            if not isinstance(value, list):
                value = [value]
            value.append(node)
        elif use_set:
            value = [node]
        else:
            value = node
        s_node[p_key] = value

    def type_coerce(self, o, coerce_type):
        if coerce_type == ID:
            if isinstance(o, URIRef):
                return self.context.shrink_iri(o)
            elif isinstance(o, BNode):
                return o.n3()
            else:
                return o
        elif coerce_type == VOCAB and isinstance(o, URIRef):
            return self.context.to_symbol(o)
        elif isinstance(o, Literal) and unicode(o.datatype) == coerce_type:
            return o
        else:
            return None

    def to_raw_value(self, graph, s, o, nodemap):
        context = self.context
        coll = self.to_collection(graph, o)
        if coll is not None:
            coll = [self.to_raw_value(graph, s, lo, nodemap)
                    for lo in self.to_collection(graph, o)]
            return {context.list_key: coll}
        elif isinstance(o, BNode):
            embed = False # TODO: self.context.active or using startnode and only one ref
            onode = self.process_subject(graph, o, nodemap)
            if onode:
                if embed and not any(s2 for s2 in graph.subjects(None, o) if s2 != s):
                    return onode
                else:
                    nodemap[onode[context.id_key]] = onode
            return {context.id_key: o.n3()}
        elif isinstance(o, URIRef):
            # TODO: embed if o != startnode (else reverse)
            return {context.id_key: context.shrink_iri(o)}
        elif isinstance(o, Literal):
            # TODO: if compact
            native = self.use_native_types and o.datatype in PLAIN_LITERAL_TYPES
            if native:
                v = o.toPython()
            else:
                v = unicode(o)
            if o.datatype:
                if native:
                    if self.context.active:
                        return v
                    else:
                        return {context.value_key: v}
                return {context.type_key: context.to_symbol(o.datatype),
                        context.value_key: v}
            elif o.language and o.language != context.language:
                return {context.lang_key: o.language,
                        context.value_key: v}
            elif not context.active or context.language and not o.language:
                return {context.value_key: v}
            else:
                return v

    def to_collection(self, graph, l):
        if l != RDF.nil and not graph.value(l, RDF.first):
            return None
        list_nodes = []
        chain = set([l])
        while l:
            if l == RDF.nil:
                return list_nodes
            if isinstance(l, URIRef):
                return None
            first, rest = None, None
            for p, o in graph.predicate_objects(l):
                if not first and p == RDF.first:
                    first = o
                elif not rest and p == RDF.rest:
                    rest = o
                elif p != RDF.type or o != RDF.List:
                    return None
            list_nodes.append(first)
            l = rest
            if l in chain:
                return None
            chain.add(l)



# -*- coding: utf-8 -*-
"""
This parser will interpret a JSON-LD document as an RDF Graph. See:

    http://json-ld.org/

Example usage::

    >>> from rdflib.plugin import register, Parser
    >>> register('json-ld', Parser, 'rdflib_jsonld.parser', 'JsonLDParser')

    >>> from rdflib import Graph, URIRef, Literal
    >>> test_json = '''
    ... {
    ...     "@context": {
    ...         "dc": "http://purl.org/dc/terms/",
    ...         "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    ...         "rdfs": "http://www.w3.org/2000/01/rdf-schema#"
    ...     },
    ...     "@id": "http://example.org/about",
    ...     "dc:title": {
    ...         "@language": "en",
    ...         "@value": "Someone's Homepage"
    ...     }
    ... }
    ... '''
    >>> g = Graph().parse(data=test_json, format='json-ld')
    >>> list(g) == [(URIRef('http://example.org/about'),
    ...     URIRef('http://purl.org/dc/terms/title'),
    ...     Literal("Someone's Homepage", lang='en'))]
    True

"""
# NOTE: This code reads the entire JSON object into memory before parsing, but
# we should consider streaming the input to deal with arbitrarily large graphs.

import warnings
from rdflib.graph import ConjunctiveGraph
from rdflib.parser import Parser, URLInputSource
from rdflib.namespace import RDF, XSD
from rdflib.term import URIRef, BNode, Literal

from .context import Context, Term, UNDEF
from .util import source_to_json, VOCAB_DELIMS, context_from_urlinputsource
from .keys import CONTEXT, GRAPH, ID, INDEX, LANG, LIST, REV, SET, TYPE, VALUE, VOCAB

__all__ = ['JsonLDParser', 'to_rdf']


TYPE_TERM = Term(unicode(RDF.type), TYPE, VOCAB)


class JsonLDParser(Parser):
    def __init__(self):
        super(JsonLDParser, self).__init__()

    def parse(self, source, sink, **kwargs):
        # TODO: docstring w. args and return value
        encoding = kwargs.get('encoding') or 'utf-8'
        if encoding not in ('utf-8', 'utf-16'):
            warnings.warn("JSON should be encoded as unicode. " +
                          "Given encoding was: %s" % encoding)

        base = kwargs.get('base') or sink.absolutize(
            source.getPublicId() or source.getSystemId() or "")
        context_data = kwargs.get('context')
        if not context_data and isinstance(source, URLInputSource):
            context_data = context_from_urlinputsource(source)
        produce_generalized_rdf = kwargs.get('produce_generalized_rdf', False)

        data = source_to_json(source)
        conj_sink = ConjunctiveGraph(
            store=sink.store, identifier=sink.identifier)
        to_rdf(data, conj_sink, base, context_data)


generalized_rdf = False
def to_rdf(data, graph, base=None, context_data=None, produce_generalized_rdf=False):
    # TODO: docstring w. args and return value
    global generalized_rdf # FIXME: not thread-safe and error-prone
    generalized_rdf = produce_generalized_rdf
    context = Context(base=base)

    if context_data:
        context.load(context_data)

    topcontext = False

    if isinstance(data, list):
        resources = data
    elif isinstance(data, dict):
        l_ctx = data.get(CONTEXT)
        if l_ctx:
            context.load(l_ctx, base)
            topcontext = True
        resources = data
        if not isinstance(resources, list):
            resources = [resources]

    if context.vocab:
        graph.bind(None, context.vocab)
    for name, term in context.terms.items():
        if term.id and term.id.endswith(VOCAB_DELIMS):
            graph.bind(name, term.id)

    for node in resources:
        _add_to_graph(graph, graph, context, node, topcontext)

    return graph


def _add_to_graph(dataset, graph, context, node, topcontext=False):
    if not isinstance(node, dict) or context.get_value(node):
        return

    if CONTEXT in node and not topcontext:
        l_ctx = node.get(CONTEXT)
        if l_ctx:
            context = context.subcontext(l_ctx)
        else:
            context = Context(base=context.doc_base)

    id_val = context.get_id(node)
    if isinstance(id_val, basestring):
        subj = _to_rdf_id(context, id_val)
    else:
        subj = BNode()

    if subj is None:
        return None

    for key, obj in node.items():
        if key in (CONTEXT, ID, context.get_key(ID)):
            continue
        if key in (REV, context.get_key(REV)):
            for rkey, robj in obj.items():
                _key_to_graph(dataset, graph, context, subj, rkey, robj, True)
        else:
            _key_to_graph(dataset, graph, context, subj, key, obj)

    return subj


def _key_to_graph(dataset, graph, context, subj, key, obj, reverse=False):

    if isinstance(obj, list):
        obj_nodes = obj
    else:
        obj_nodes = [obj]

    term = context.terms.get(key)
    if term:
        term_id = term.id
        if term.container == LIST:
            obj_nodes = [{LIST: obj_nodes}]
        elif isinstance(obj, dict):
            if term.container == INDEX:
                obj_nodes = []
                for values in obj.values():
                    if not isinstance(values, list):
                        obj_nodes.append(values)
                    else:
                        obj_nodes += values
            elif term.container == LANG:
                obj_nodes = []
                for lang, values in obj.items():
                    if not isinstance(values, list):
                        values = [values]
                    for v in values:
                        obj_nodes.append((v, lang))
    else:
        term_id = None

    if TYPE in (key, term_id):
        term = TYPE_TERM
    elif GRAPH in (key, term_id):
        #assert graph.context_aware
        subgraph = dataset.get_context(subj)
        for onode in obj_nodes:
            _add_to_graph(dataset, subgraph, context, onode)
        return
    elif SET in (key, term_id):
        for onode in obj_nodes:
            _add_to_graph(dataset, graph, context, onode)
        return

    pred_uri = term.id if term else context.expand(key)

    flattened = []
    for obj in obj_nodes:
        if isinstance(obj, dict):
            objs = context.get_set(obj)
            if objs is not None:
                obj = objs
        if isinstance(obj, list):
            flattened += obj
            continue
        flattened.append(obj)
    obj_nodes = flattened

    if not pred_uri:
        return

    if term and term.reverse:
        reverse = not reverse

    bid = _get_bnodeid(pred_uri)
    if bid:
        if not generalized_rdf:
            return
        pred = BNode(bid)
    else:
        pred = URIRef(pred_uri)
    for obj_node in obj_nodes:
        obj = _to_object(dataset, graph, context, term, obj_node)
        if obj is None:
            continue
        if reverse:
            graph.add((obj, pred, subj))
        else:
            graph.add((subj, pred, obj))


def _to_object(dataset, graph, context, term, node, inlist=False):

    if node is None:
        return

    if isinstance(node, tuple):
        value, lang = node
        if value is None:
            return
        return Literal(value, lang=lang)

    if isinstance(node, dict):
        node_list = context.get_list(node)
        if node_list is not None:
            if inlist: # TODO: and NO_LISTS_OF_LISTS
                return
            listref = _add_list(dataset, graph, context, term, node_list)
            if listref:
                return listref

    else: # expand..
        if not term or not term.type:
            if isinstance(node, float):
                return Literal(node, datatype=XSD.double)
            if term and term.language is not UNDEF:
                lang = term.language
            else:
                lang = context.language
            return Literal(node, lang=lang)
        else:
            if term.type == ID:
                node = {ID: context.resolve(node)}
            elif term.type == VOCAB:
                node = {ID: context.expand(node) or context.resolve_iri(node)}
            else:
                node = {TYPE: term.type,
                        VALUE: node}

    lang = context.get_language(node)
    if lang or context.get_key(VALUE) in node or VALUE in node:
        value = context.get_value(node)
        if value is None:
            return None
        datatype = not lang and context.get_type(node) or None
        if lang:
            return Literal(value, lang=lang)
        elif datatype:
            return Literal(value, datatype=context.expand(datatype))
        else:
            return Literal(value)
    else:
        return _add_to_graph(dataset, graph, context, node)


def _to_rdf_id(context, id_val):
    bid = _get_bnodeid(id_val)
    if bid:
        return BNode(bid)
    else:
        uri = context.resolve(id_val)
        if not generalized_rdf and ':' not in uri:
            return None
        return URIRef(uri)


def _get_bnodeid(ref):
    if not ref.startswith('_:'):
        return
    bid = ref.split('_:', 1)[-1]
    return bid or None


def _add_list(dataset, graph, context, term, node_list):
    if not isinstance(node_list, list):
        node_list = [node_list]
    first_subj = BNode()
    subj, rest = first_subj, None
    for node in node_list:
        if node is None:
            continue
        if rest:
            graph.add((subj, RDF.rest, rest))
            subj = rest
        obj = _to_object(dataset, graph, context, term, node, inlist=True)
        if obj is None:
            continue
        graph.add((subj, RDF.first, obj))
        rest = BNode()
    if rest:
        graph.add((subj, RDF.rest, RDF.nil))
        return first_subj
    else:
        return RDF.nil



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import re

# Ridiculous as it may seem, we need to import multiprocessing and logging here
# in order to get tests to pass smoothly on python 2.7.
try:
    import multiprocessing
    import logging
except:
    pass


def setup_python3():
    # Taken from "distribute" setup.py
    from distutils.filelist import FileList
    from distutils import dir_util, file_util, util, log
    from os.path import join

    tmp_src = join("build", "src")
    log.set_verbosity(1)
    fl = FileList()
    for line in open("MANIFEST.in"):
        if not line.strip():
            continue
        fl.process_template_line(line)
    dir_util.create_tree(tmp_src, fl.files)
    outfiles_2to3 = []
    for f in fl.files:
        outf, copied = file_util.copy_file(f, join(tmp_src, f), update=1)
        if copied and outf.endswith(".py"):
            outfiles_2to3.append(outf)

    util.run_2to3(outfiles_2to3)

    # arrange setup to use the copy
    sys.path.insert(0, tmp_src)

    return tmp_src


# Find version. We have to do this because we can't import it in Python 3 until
# its been automatically converted in the setup process.
def find_version(filename):
    _version_re = re.compile(r'__version__ = "(.*)"')
    for line in open(filename):
        version_match = _version_re.match(line)
        if version_match:
            return version_match.group(1)

__version__ = find_version('rdflib_jsonld/__init__.py')

install_requires = ["rdflib>=4.0", ] 

if sys.version_info[:2] < (2, 6):
    install_requires += ["simplejson"]

config = dict(
    name = 'rdflib-jsonld',
    description = "rdflib extension adding JSON-LD parser and serializer",
    maintainer = "RDFLib Team",
    maintainer_email = "http://groups.google.com/group/rdflib-dev",
    url = "https://github.com/RDFLib/rdflib-jsonld",
    version = __version__,
    download_url = "https://github.com/RDFLib/rdflib-jsonld/zipball/master",
    license = "BSD",
    platforms = ["any"],
    long_description = \
    """
    This parser/serialiser will

    * read in an JSON-LD formatted document and create an RDF graph
    * serialize an RDF graph to JSON-LD formatted output

    See:

        http://json-ld.org/
    """,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "License :: OSI Approved :: BSD License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Operating System :: OS Independent",
        "Natural Language :: English",
                   ],
    packages = ["rdflib_jsonld"],
    test_suite = "test",
    install_requires = ["rdflib>=4.2"],
    entry_points = {
        'rdf.plugins.parser': [
            'json-ld = rdflib_jsonld.parser:JsonLDParser',
        ],
        'rdf.plugins.serializer': [
            'json-ld = rdflib_jsonld.serializer:JsonLDSerializer',
        ],
    }
)

if sys.version_info[0] >= 3:
    from setuptools import setup
    config.update({'use_2to3': True})
    config.update({'src_root': setup_python3()})
else:
    try:
        from setuptools import setup
        config.update({'test_suite' : "nose.collector"})
    except ImportError:
        from distutils.core import setup


setup(**config)



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals
import re
import json
import itertools
from rdflib import Graph
from rdflib.serializer import Serializer


cases = []
def case(*args): cases.append(args)


case("""
@prefix dc: <http://purl.org/dc/terms/> .
<http://example.org/>
    dc:title "Homepage"@en .
""",
{
    "@context": {"@vocab": "http://purl.org/dc/terms/", "@language": "en"},
    "@id": "http://example.org/",
    "title": "Homepage"
}
)


case("""
@prefix dc: <http://purl.org/dc/terms/> .
<http://example.org/>
    dc:title "Homepage"@en, "Hemsida"@sv .
""",
{
    "@context": {"@vocab": "http://purl.org/dc/terms/", "title": {"@container": "@language"}},
    "@id": "http://example.org/",
    "title": {"en": "Homepage", "sv": "Hemsida"}
}
)


case("""
@prefix dc: <http://purl.org/dc/terms/> .
<http://example.org/>
    dc:title "Homepage"@en, "Hemsida"@sv .
""",
{
    "@context": {
        "@vocab": "http://purl.org/dc/terms/",
        "@language": "sv",
        "title_en": {"@id": "title", "@language": "en"}
    },
    "@id": "http://example.org/",
    "title_en": "Homepage",
    "title": "Hemsida"
}
)


# .. Requires set values to be sorted to be predicatble
#case("""
#@prefix dc: <http://purl.org/dc/terms/> .
#<http://example.org/>
#    dc:title "Homepage"@en, "Home Page"@en, "Home Page"@en-GB, "Hemsida"@sv .
#""",
#{
#    "@context": "-||-",
#    "@id": "http://example.org/",
#    "title_en": ["Homepage", "Home Page"],
#    "title": [{"@language": "en-GB", "@value": "Home Page"}, "Hemsida"]
#}
#)


case("""
@prefix dc: <http://purl.org/dc/terms/> .
<http://example.org/easter_island>
    dc:title "Påskön"@sv .
""",
{
    "@context": {"@vocab": "http://purl.org/dc/terms/", "@language": "sv"},
    "@id": "http://example.org/easter_island",
    "title": "Påskön"
}
)


case("""
@prefix : <http://example.org/ns#> .
<http://example.org/> :has _:blank-1 .
""",
{
    "@context": {"has": {"@type": "@id", "@id": "http://example.org/ns#has"}},
    "@id": "http://example.org/",
    "has": "_:blank-1"
}
)


case("""
@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
@prefix : <http://example.org/ns#> .
:Something rdfs:subClassOf :Thing .
""",
{
    "@context": {
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "v": "http://example.org/ns#",
        "rdfs:subClassOf": {"@container": "@set"}
    },
    "@id": "v:Something",
    "rdfs:subClassOf": [{"@id": "v:Thing"}]
}
)


case("""
@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
@prefix : <http://example.org/ns#> .
:Something rdfs:subClassOf :Thing .
""",
{
    "@context": {
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "v": "http://example.org/ns#",
        "subClassOf": {"@id": "rdfs:subClassOf", "@type": "@id", "@container": "@set"}
    },
    "@id": "v:Something",
    "subClassOf": ["v:Thing"]
}
)


case("""
@prefix owl: <http://www.w3.org/2002/07/owl#> .
@prefix : <http://example.org/ns#> .
:World owl:unionOf (:Everyhing :Nothing) .
""",
{
    "@context": {
        "owl": "http://www.w3.org/2002/07/owl#",
        "v": "http://example.org/ns#"
    },
    "@id": "v:World",
    "owl:unionOf": {"@list": [{"@id": "v:Everyhing"}, {"@id": "v:Nothing"}]}
}
)


case("""
@prefix owl: <http://www.w3.org/2002/07/owl#> .
@prefix : <http://example.org/ns#> .
:World owl:unionOf (:Everyhing :Nothing) .
""",
{
    "@context": {
        "owl": "http://www.w3.org/2002/07/owl#",
        "v": "http://example.org/ns#",
        "unionOf": {"@id": "owl:unionOf", "@container": "@list"}
    },
    "@id": "v:World",
    "unionOf": [{"@id": "v:Everyhing"}, {"@id": "v:Nothing"}]
}
)


case("""
@prefix owl: <http://www.w3.org/2002/07/owl#> .
@prefix : <http://example.org/ns#> .
:World owl:unionOf (:Everyhing :Nothing) .
""",
{
    "@context": {
        "owl": "http://www.w3.org/2002/07/owl#",
        "v": "http://example.org/ns#",
        "unionOf": {"@id": "owl:unionOf", "@type": "@id", "@container": "@list"}
    },
    "@id": "v:World",
    "unionOf": ["v:Everyhing", "v:Nothing"]
}
)


json_kwargs = dict(indent=2, separators=(',', ': '), sort_keys=True, ensure_ascii=False)

def run(data, expected):
    context = expected['@context']
    g = Graph().parse(data=data, format='turtle')
    result = g.serialize(format='json-ld', context=context, **json_kwargs).decode('utf-8')
    incr = itertools.count(1)
    result = re.sub(r'"_:[^"]+"', lambda m: '"_:blank-%s"' % incr.next(), result)
    expected = json.dumps(expected, **json_kwargs)
    assert result == expected


def test_cases():
    for data, expected in cases:
        yield run, data, expected
